'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var React = require('react');
var createCache = _interopDefault(require('@emotion/cache'));
var utils = require('@emotion/utils');
var serialize = require('@emotion/serialize');
var sheet = require('@emotion/sheet');

function _inheritsLoose(subClass, superClass) {
  subClass.prototype = Object.create(superClass.prototype);
  subClass.prototype.constructor = subClass;
  subClass.__proto__ = superClass;
}

var BasicProvider =
/*#__PURE__*/
function (_React$Component) {
  _inheritsLoose(BasicProvider, _React$Component);

  function BasicProvider(props) {
    var _this;

    _this = _React$Component.call(this, props) || this;
    _this.state = {
      value: createCache()
    };
    return _this;
  }

  var _proto = BasicProvider.prototype;

  _proto.render = function render() {
    return React.createElement(CSSContext.Provider, this.state, this.props.children(this.state.value));
  };

  return BasicProvider;
}(React.Component);

var CSSContext = React.createContext(process.env.NODE_ENV !== 'test' ? createCache() : null);
function withCSSContext(func) {
  return function (props) {
    return React.createElement(CSSContext.Consumer, null, function (context) {
      if (context === null) {
        return React.createElement(BasicProvider, null, function (newContext) {
          return func(props, newContext);
        });
      } else {
        return func(props, context);
      }
    });
  };
}
function consume(func) {
  return React.createElement(CSSContext.Consumer, null, function (context) {
    if (context === null) {
      return React.createElement(BasicProvider, null, function (newContext) {
        return func(newContext);
      });
    } else {
      return func(context);
    }
  });
}

var jsx = function jsx(type, props) {
  var _arguments = arguments;

  // $FlowFixMe
  if (props == null || props.css == null || type.__emotion_component === true) {
    // $FlowFixMe
    return React.createElement.apply(undefined, arguments);
  }

  if (typeof props.css === 'string' && // check if there is a css declaration
  props.css.indexOf(':') !== -1 && process.env.NODE_ENV !== 'production') {
    throw new Error("Strings are not allowed as css prop values, please wrap it in a css template literal from '@emotion/css' like this: css`" + props.css + "`");
  }

  return consume(function (context) {
    var registeredStyles = [];
    var className = '';

    if (props.className !== undefined) {
      className = utils.getRegisteredStyles(context.registered, registeredStyles, props.className);
    }

    registeredStyles.push(typeof props.css === 'function' ? props.css(context.theme) : props.css);
    var serialized = serialize.serializeStyles(context.registered, registeredStyles);
    var rules = utils.insertStyles(context, serialized);
    className += context.key + "-" + serialized.name;
    var newProps = {};

    for (var key in props) {
      if (Object.prototype.hasOwnProperty.call(props, key) && key !== 'css') {
        newProps[key] = props[key];
      }
    }

    newProps.className = className;
    var argsLength = _arguments.length;
    var createElementArgArray = new Array(argsLength);
    createElementArgArray[0] = type;
    createElementArgArray[1] = newProps;

    for (var i = 2; i < argsLength; i++) {
      createElementArgArray[i] = _arguments[i];
    } // $FlowFixMe


    var ele = React.createElement.apply(undefined, createElementArgArray);

    return ele;
  });
};

var Global =
/*#__PURE__*/
function (_React$Component) {
  _inheritsLoose(Global, _React$Component);

  function Global(props) {
    var _this;

    _this = _React$Component.call(this, props) || this;

    _this.renderChild = function (context) {
      var serialized = serialize.serializeStyles(context.registered, [_this.props.css]);

      if (_this.oldName !== serialized.name) {
        {
          _this.sheet = new sheet.StyleSheet({
            key: context.key + "-global",
            // $FlowFixMe
            nonce: context.sheet.nonce,
            container: context.sheet.container
          });
        }

        _this.oldName = serialized.name;
        var rules = context.stylis("", serialized.styles);

        {
          _this.sheet.flush();

          rules.forEach(function (rule) {
            _this.sheet.insert(rule);
          });
        }
      }

      return null;
    };

    return _this;
  }

  var _proto = Global.prototype;

  _proto.componentWillUnmount = function componentWillUnmount() {
    this.sheet.flush();
  };

  _proto.render = function render() {
    return consume(this.renderChild);
  };

  Global.__emotion_component = true;
  return Global;
}(React.Component);

exports.jsx = jsx;
exports.Global = Global;
exports.CSSContext = CSSContext;
exports.withCSSContext = withCSSContext;
exports.consume = consume;
//# sourceMappingURL=index.browser.cjs.js.map
